@@ -115,6 +115,8 @@ def categorical_scatter(
             _data['hue'] = data[hue]
         else:
             _data['hue'] = hue
+        _data['hue'] = _data['hue'].astype('category').cat.remove_unused_categories()
+
         hue = 'hue'
         _data['hue'] = _data['hue'].astype('category')
         # deal with color palette
@@ -137,7 +139,7 @@ def categorical_scatter(
                     **_scatter_kws)
 
     # deal with text annotation
-    if text_anno:
+    if text_anno is not None:
         if isinstance(text_anno, str):
             _data['text_anno'] = data[text_anno]
         else:
